from typing import List, Callable
from vessel.pipeline import Step, Payload
from vessel.steps import VaultBaseStep
from vessel.logging import logger
import subprocess
import hvac
import base64
from config import Config
import os.path

class KubebenchStep(VaultBaseStep):
  """
  Unseal Vault Step
  """
  def get_rw_token(self, data):
    return base64.b64decode(data['private']['rw-token']).decode("utf-8")

  def __init__(self, vault_host):
    super().__init__(vault_host)
    #self.unlocked = unlocked

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn('Taking Vault root token')
    
    keys = self.get_vault_keys(payload, start_fn, prompt_fn)
    root_token = keys['root_token']
    self.vault.token = keys["root_token"]
    cluster_token = payload.token

    start_fn(f"Cluster token is {cluster_token}. Taking k8s auth token from Vault")

    unlock_pub_secrets = self.vault.secrets.kv.v2.read_secret_version(
      path=cluster_token + "-daas-rw-ops",
      mount_point="daas_public"
    )

    pub_data = unlock_pub_secrets['data']['data']

    auth_token = pub_data['token']
    sa_name = pub_data['sa-name']
    distribution = pub_data['distribution']
    namespace = pub_data['namespace']

    start_fn(f"Cluster distribution of {cluster_token} is {distribution}")
    start_fn(f"Using the service account {sa_name} for installing kube-bench into the namespace {namespace}")

    if distribution == 'kubernetes':
      job_path = os.path.join(Config.KUBEBENCH_YAML_BASE, Config.KUBEBENCH_YAML_K8S)
      bashCommand = f"kubectl -n {namespace} apply -f {job_path} --token={auth_token}"

    elif distribution == 'openshift':
      job_path = os.path.join(Config.KUBEBENCH_YAML_BASE, Config.KUBEBENCH_YAML_OCP_311)
      bashCommand = f"oc -n {namespace} apply -f {job_path} --token={auth_token}"
  
    start_fn(f"Kube Bench yaml job path is {job_path}")
    process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
    output, error = process.communicate()
    if error:
      raise Exception(error)
    for out in output.decode('utf-8').split("\n"):
      end_fn(out)

    end_fn(f"Job {job_path} applied!")


import os
import json
import kubernetes
from urllib.parse import urljoin
from vessel.pipeline import Step, Payload
from typing import List, Callable, Tuple
from vessel.logging import logger
from vessel.utilities import sanitizeClusterName

class UpdateStep(Step):
  """
  Deploy Agent
  """
  def __init__(self, agent_tag, sentinel_tag):
    super().__init__()
    self.agent_tag = agent_tag
    self.sentinel_tag = sentinel_tag


  def patch_image(self, component, deployment, namespace, image, kube_apps):
    try:
        body = {
          "spec": {
            "template": {
              "spec": {
                "containers": [
                  {
                    "name": deployment,
                    "image": image
                    }
                ]
              }
            }
          }
        }
        api_response = kube_apps.patch_namespaced_deployment(deployment, namespace, body)
    except ApiException as e:
        print("Exception when calling AppsV1Api->patch_namespaced_deployment: %s\n" % e)


  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    
    with open(os.path.expanduser('~/.vessel/devops.json')) as f:
      devops = json.load(f)

    with open(os.path.expanduser(f"~/.daas/{payload.token}/registration.json"), 'r') as f:
      registration = json.load(f)

    namespace = devops['vessel_namespace']
    img_base = devops['images_base_url']
    k8s_token = devops['devops_sa_token']
    cluster_name = sanitizeClusterName(registration['cluster']['result']['name'])
    
    # Init kube client
    start_fn('Creating kubernetes client configuration...')
    configuration = kubernetes.client.Configuration()
    configuration.host = devops['kubernetes_url']
    configuration.verify_ssl = False
    configuration.api_key = {"authorization": "Bearer " + k8s_token}
    api_client = kubernetes.client.ApiClient(configuration)
    apps = kubernetes.client.AppsV1Api(api_client)

    agent_deployment = cluster_name + '-agent'
    sentinel_deployment = cluster_name + '-sentinel'
    running_agent = apps.read_namespaced_deployment(agent_deployment, namespace).spec.template.spec.containers[0].image
    start_fn('At moment, you are running this tag image for Vessel Agent => ' + running_agent.split(":")[-1] )
    running_sentinel = apps.read_namespaced_deployment(sentinel_deployment, namespace).spec.template.spec.containers[0].image
    start_fn('At moment, you are running this tag image for Vessel Sentinel => ' + running_sentinel.split(":")[-1] )

    if running_agent.split(":")[-1] != self.agent_tag:
      end_fn('Updating Agent to tag ' + self.agent_tag)
      image = urljoin(img_base, 'workstation-agent') + ':' + self.agent_tag
      self.patch_image('Agent', agent_deployment, namespace, image, apps)
    else:
      end_fn('Agent is already updated')

    if running_sentinel.split(":")[-1] != self.sentinel_tag: 
      end_fn('Updating Sentinel to tag ' + self.sentinel_tag)
      image = urljoin(img_base, 'workstation-sentinel') + ':' + self.sentinel_tag
      self.patch_image('Sentinel', sentinel_deployment, namespace, image, apps)
    else:
      end_fn('Sentinel is already updated')